import _defineProperty from 'babel-runtime/helpers/defineProperty';
import _extends from 'babel-runtime/helpers/extends';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _createClass from 'babel-runtime/helpers/createClass';
import _possibleConstructorReturn from 'babel-runtime/helpers/possibleConstructorReturn';
import _inherits from 'babel-runtime/helpers/inherits';
import * as React from 'react';
import classNames from 'classnames';
import isFunction from 'lodash/isFunction';
import Animate from '../rc-components/animate';
import Icon from '../icon'; // add file-type icon, modified by yang.wang06@hand-china.com
import Tooltip from '../tooltip';
import PopConfirm from '../popconfirm';
import Progress from '../progress';
import '../icon/style'; // modified by yang.wang06@hand-china.com
import '../tooltip/style'; // modified by yang.wang06@hand-china.com
import '../popconfirm/style'; // modified by yang.wang06@hand-china.com
import '../progress/style'; // modified by yang.wang06@hand-china.com
// https://developer.mozilla.org/en-US/docs/Web/API/FileReader/readAsDataURL
var previewFile = function previewFile(file, callback) {
    var reader = new FileReader();
    reader.onloadend = function () {
        return callback(reader.result);
    };
    reader.readAsDataURL(file);
};
var extname = function extname(url) {
    if (!url) {
        return '';
    }
    var temp = url.split('/');
    var filename = temp[temp.length - 1];
    var filenameWithoutSuffix = filename.split(/#|\?/)[0];
    return (/\.[^./\\]*$/.exec(filenameWithoutSuffix) || [''])[0];
};
var isImageUrl = function isImageUrl(url) {
    var extension = extname(url);
    if (/^data:image\//.test(url) || /(webp|svg|png|gif|jpg|jpeg|bmp)$/i.test(extension)) {
        return true;
    } else if (/^data:/.test(url)) {
        // other file types of base64
        return false;
    } else if (extension) {
        // other file types which have extension
        return false;
    }
    return true;
};
/**
 * getFileIconByFileName, modified by yang.wang06@hand-china.com
 * get file-type icon
 * @param {string?} fileName
 * @param {string?} prefixCls
 */
function getFileIconByFileName(fileName, prefixCls) {
    if (fileName) {
        var fA = fileName.split('.');
        var fileExt = fA && fA[fA.length - 1];
        var iconType = 'file-unknown';
        switch (fileExt) {
            case 'doc':
            case 'docx':
                iconType = 'file-word';
                break;
            case 'xls':
            case 'xlsx':
                iconType = 'file-excel';
                break;
            case 'ppt':
            case 'pptx':
                iconType = 'file-ppt';
                break;
            case 'pdf':
                iconType = 'file-pdf';
                break;
            case 'txt':
                iconType = 'file-text';
                break;
            case 'md':
                iconType = 'file-markdown';
            default:
                break;
        }
        return React.createElement(Icon, { type: iconType, className: prefixCls + '-list-item-icon' });
    }
    return React.createElement(Icon, { type: 'file-unknown', className: prefixCls + '-list-item-icon' });
}

var UploadList = function (_React$Component) {
    _inherits(UploadList, _React$Component);

    function UploadList() {
        _classCallCheck(this, UploadList);

        var _this = _possibleConstructorReturn(this, (UploadList.__proto__ || Object.getPrototypeOf(UploadList)).apply(this, arguments));

        _this.handleClose = function (file) {
            var onRemove = _this.props.onRemove;

            if (onRemove) {
                onRemove(file);
            }
        };
        _this.handlePreview = function (file, e) {
            var onPreview = _this.props.onPreview;

            if (!onPreview) {
                return;
            }
            e.preventDefault();
            return onPreview(file);
        };
        /**
         * modified by yang.wang06@hand-china.com
         * @param {UploadFile} file
         * @param {React.SyntheticEvent<HTMLElement>} e
         */
        _this.handleReUpload = function (file, e) {
            var onReUpload = _this.props.onReUpload;

            if (!onReUpload) {
                return;
            }
            e.preventDefault();
            return onReUpload(file);
        };
        return _this;
    }

    _createClass(UploadList, [{
        key: 'componentDidUpdate',
        value: function componentDidUpdate() {
            var _this2 = this;

            if (this.props.listType !== 'picture' && this.props.listType !== 'picture-card') {
                return;
            }
            (this.props.items || []).forEach(function (file) {
                if (typeof document === 'undefined' || typeof window === 'undefined' || !window.FileReader || !window.File || !(file.originFileObj instanceof File) || file.thumbUrl !== undefined) {
                    return;
                }
                /*eslint-disable */
                file.thumbUrl = '';
                /*eslint-enable */
                previewFile(file.originFileObj, function (previewDataUrl) {
                    /*eslint-disable */
                    file.thumbUrl = previewDataUrl;
                    /*eslint-enable */
                    _this2.forceUpdate();
                });
            });
        }
    }, {
        key: 'render',
        value: function render() {
            var _this3 = this,
                _classNames2;

            var _props = this.props,
                prefixCls = _props.prefixCls,
                _props$items = _props.items,
                items = _props$items === undefined ? [] : _props$items,
                listType = _props.listType,
                showPreviewIcon = _props.showPreviewIcon,
                showRemoveIcon = _props.showRemoveIcon,
                showDownloadIcon = _props.showDownloadIcon,
                showReUploadIcon = _props.showReUploadIcon,
                removePopConfirmTitle = _props.removePopConfirmTitle,
                reUploadText = _props.reUploadText,
                reUploadPopConfirmTitle = _props.reUploadPopConfirmTitle,
                locale = _props.locale,
                getCustomFilenameTitle = _props.getCustomFilenameTitle;

            var list = items.map(function (file) {
                var _classNames;

                var progress = void 0;
                var icon = React.createElement(Icon, { type: file.status === 'uploading' ? 'loading' : 'paper-clip' });
                // combine stat to stat, modified by yang.wang06@hand-china.com
                var stat = {
                    // modified by yang.wang06@hand-china.com
                    isImg: isImageUrl(file.name || file.thumbUrl || file.url),
                    isUploading: file.status === 'uploading',
                    isError: file.status === 'error',
                    isPictureCard: listType === 'picture-card'
                };
                // add an file-name download, modified by yang.wang06@hand-china.com
                var downloadLinkProps = _extends({}, file.linkProps, { rel: 'noopener noreferrer' });
                if (stat.isError) {
                    downloadLinkProps.style = { color: '#f5222d' };
                }
                if (!(stat.isError || stat.isUploading)) {
                    downloadLinkProps.href = file.url || file.thumbUrl;
                    downloadLinkProps.target = '_blank';
                    if (downloadLinkProps.href) {
                        downloadLinkProps.filename = file.name;
                    }
                }
                if (listType === 'picture' || stat.isPictureCard) {
                    if (stat.isPictureCard && stat.isUploading) {
                        icon = React.createElement(
                            'div',
                            { className: prefixCls + '-list-item-uploading-text' },
                            locale.uploading
                        );
                    } else if (!file.thumbUrl && !file.url) {
                        icon = React.createElement(Icon, { className: prefixCls + '-list-item-thumbnail', type: 'picture' });
                    } else {
                        var thumbnail = stat.isImg ? React.createElement('img', { src: file.thumbUrl || file.url, alt: file.name }) : getFileIconByFileName(file.name, prefixCls); // file-type icon, modified by yang.wang06@hand-china.com
                        icon = React.createElement(
                            'a',
                            { className: prefixCls + '-list-item-thumbnail', onClick: function onClick(e) {
                                    return _this3.handlePreview(file, e);
                                }, href: file.url || file.thumbUrl, target: '_blank', rel: 'noopener noreferrer' },
                            thumbnail
                        );
                    }
                }
                if (stat.isUploading) {
                    // show loading icon if upload progress listener is disabled
                    var loadingProgress = 'percent' in file ? React.createElement(Progress, _extends({ type: 'line' }, _this3.props.progressAttr, { percent: file.percent })) : null;
                    progress = React.createElement(
                        'div',
                        { className: prefixCls + '-list-item-progress', key: 'progress' },
                        loadingProgress
                    );
                }
                var infoUploadingClass = classNames((_classNames = {}, _defineProperty(_classNames, prefixCls + '-list-item', true), _defineProperty(_classNames, prefixCls + '-list-item-' + file.status, true), _classNames));
                var preview = file.url ? React.createElement(
                    'a',
                    _extends({}, file.linkProps, { href: file.url, target: '_blank', rel: 'noopener noreferrer', className: prefixCls + '-list-item-name', onClick: function onClick(e) {
                            return _this3.handlePreview(file, e);
                        }, title: file.name }),
                    file.name
                ) : React.createElement(
                    'span',
                    { className: prefixCls + '-list-item-name', onClick: function onClick(e) {
                            return _this3.handlePreview(file, e);
                        }, title: file.name },
                    file.name
                );
                var style = {
                    pointerEvents: 'none',
                    opacity: 0.5
                };
                /* previewIcon support function, modified by yang.wang06@hand-china.com */
                var previewIcon = (isFunction(showPreviewIcon) ? showPreviewIcon(file) :
                /* only when file is img has preview, modified by yang.wang06@hand-china.com */
                stat.isImg && showPreviewIcon) ? React.createElement(
                    'a',
                    { href: file.url || file.thumbUrl, target: '_blank', rel: 'noopener noreferrer', style: file.url || file.thumbUrl ? undefined : style, onClick: function onClick(e) {
                            return _this3.handlePreview(file, e);
                        }, title: locale.previewFile },
                    React.createElement(Icon, { type: 'eye-o' })
                ) : null;
                var reUploadIcon = (isFunction(showReUploadIcon) ? showReUploadIcon(file) : showReUploadIcon) ? React.createElement(
                    PopConfirm,
                    { title: reUploadPopConfirmTitle, onConfirm: function onConfirm(e) {
                            _this3.handleReUpload(file, e);
                        } },
                    React.createElement(Icon, { type: 'upload', title: reUploadText })
                ) : null;
                var removeIcon = showRemoveIcon ? React.createElement(
                    PopConfirm,
                    { title: removePopConfirmTitle, onConfirm: function onConfirm() {
                            _this3.handleClose(file);
                        } },
                    React.createElement(Icon, { type: 'delete', className: prefixCls + '-list-item-action-remove', title: locale.removeFile })
                ) : null;
                var removeIconCross = showRemoveIcon ? React.createElement(Icon, { type: 'cross', title: locale.removeFile, onClick: function onClick() {
                        return _this3.handleClose(file);
                    } }) : null;
                var downloadIcon = (isFunction(showDownloadIcon) ? showDownloadIcon(file) : showDownloadIcon) && !stat.isError && !stat.isUploading && React.createElement(
                    'a',
                    downloadLinkProps,
                    React.createElement(Icon, { type: 'download' })
                );
                // when error action can only be remove ,modified by yang.wang06@hand-china.com
                var actions = stat.isPictureCard && !stat.isUploading ? React.createElement(
                    'span',
                    { className: prefixCls + '-list-item-actions' },
                    stat.isError || previewIcon,
                    downloadIcon,
                    reUploadIcon
                ) : removeIconCross;
                var message = void 0;
                if (file.response && typeof file.response === 'string') {
                    message = file.response;
                } else {
                    message = file.error && file.error.statusText || locale.uploadError;
                }
                // add className `${prefixCls}-list-item-picture-card` for file-name's height
                // ,modified by yang.wang06@hand-china.com
                var iconAndPreview = stat.isError ? React.createElement(
                    Tooltip,
                    { title: message },
                    React.createElement(
                        'span',
                        { className: prefixCls + '-list-item-picture-card' },
                        icon,
                        stat.isPictureCard || preview
                    )
                ) : React.createElement(
                    'span',
                    { className: prefixCls + '-list-item-picture-card' },
                    icon,
                    stat.isPictureCard || preview
                );
                // file-name ,modified by yang.wang06@hand-china.com
                var fileName = downloadLinkProps.href ? React.createElement(
                    'a',
                    _extends({ className: prefixCls + '-list-item-file-name' }, downloadLinkProps),
                    file.name
                ) : React.createElement(
                    'span',
                    _extends({ className: prefixCls + '-list-item-file-name' }, downloadLinkProps),
                    file.name
                );
                fileName = React.createElement(
                    Tooltip,
                    { title: getCustomFilenameTitle ? getCustomFilenameTitle(file) : file.name, placement: 'bottom' },
                    fileName
                );
                fileName = stat.isPictureCard && !stat.isUploading ? fileName : null;
                return React.createElement(
                    'div',
                    { className: infoUploadingClass, key: file.uid },
                    React.createElement(
                        'div',
                        { className: prefixCls + '-list-item-info' },
                        iconAndPreview,
                        fileName
                    ),
                    actions,
                    stat.isPictureCard && removeIcon,
                    React.createElement(
                        Animate,
                        { transitionName: 'fade', component: '' },
                        progress
                    )
                );
            });
            var listClassNames = classNames((_classNames2 = {}, _defineProperty(_classNames2, prefixCls + '-list', true), _defineProperty(_classNames2, prefixCls + '-list-' + listType, true), _classNames2));
            var animationDirection = listType === 'picture-card' ? 'animate-inline' : 'animate';
            return React.createElement(
                Animate,
                { transitionName: prefixCls + '-' + animationDirection, component: 'div', className: listClassNames },
                list
            );
        }
    }]);

    return UploadList;
}(React.Component);

export default UploadList;

UploadList.defaultProps = {
    listType: 'text',
    progressAttr: {
        strokeWidth: 2,
        showInfo: false
    },
    prefixCls: 'ant-upload',
    showRemoveIcon: true,
    showPreviewIcon: true,
    showDownloadIcon: true,
    showReUploadIcon: false
};